@@ -1,5 +1,6 @@
+import pytest
+
 import networkx as nx
-from networkx.testing import almost_equal
 
 
 def example1a_G():
@@ -36,16 +37,16 @@ class TestPercolationCentrality:
         G = example1a_G()
         p = nx.percolation_centrality(G)
         p_answer = {4: 0.625, 6: 0.667}
-        for n in p_answer:
-            assert almost_equal(p[n], p_answer[n], places=3)
+        for n, k in p_answer.items():
+            assert p[n] == pytest.approx(k, abs=1e-3)
 
     def test_percolation_example1b(self):
         """percolation centrality: example 1a"""
         G = example1b_G()
         p = nx.percolation_centrality(G)
         p_answer = {4: 0.825, 6: 0.4}
-        for n in p_answer:
-            assert almost_equal(p[n], p_answer[n], places=3)
+        for n, k in p_answer.items():
+            assert p[n] == pytest.approx(k, abs=1e-3)
 
     def test_converge_to_betweenness(self):
         """percolation centrality: should converge to betweenness
@@ -73,9 +74,9 @@ class TestPercolationCentrality:
         p_states = {k: 1.0 for k, v in b_answer.items()}
         p_answer = nx.percolation_centrality(G, states=p_states)
         for n in sorted(G):
-            assert almost_equal(p_answer[n], b_answer[n], places=3)
+            assert p_answer[n] == pytest.approx(b_answer[n], abs=1e-3)
 
         p_states = {k: 0.3 for k, v in b_answer.items()}
         p_answer = nx.percolation_centrality(G, states=p_states)
         for n in sorted(G):
-            assert almost_equal(p_answer[n], b_answer[n], places=3)
+            assert p_answer[n] == pytest.approx(b_answer[n], abs=1e-3)
