@@ -1,11 +1,12 @@
-from itertools import chain
-from itertools import combinations
+from itertools import chain, combinations
 
 import pytest
 
 import networkx as nx
-from networkx.algorithms.community import label_propagation_communities
-from networkx.algorithms.community import asyn_lpa_communities
+from networkx.algorithms.community import (
+    asyn_lpa_communities,
+    label_propagation_communities,
+)
 
 
 def test_directed_not_supported():
@@ -18,6 +19,14 @@ def test_directed_not_supported():
         result = label_propagation_communities(test)
 
 
+def test_iterator_vs_iterable():
+    G = nx.empty_graph("a")
+    assert list(label_propagation_communities(G)) == [{"a"}]
+    for community in label_propagation_communities(G):
+        assert community == {"a"}
+    pytest.raises(TypeError, next, label_propagation_communities(G))
+
+
 def test_one_node():
     test = nx.Graph()
     test.add_node("a")
