@@ -4,17 +4,18 @@
 import pytest
 
 import networkx as nx
-from networkx.classes import Graph
-from networkx.classes import MultiDiGraph
-from networkx.generators.directed import gn_graph
-from networkx.generators.directed import gnr_graph
-from networkx.generators.directed import gnc_graph
-from networkx.generators.directed import random_k_out_graph
-from networkx.generators.directed import random_uniform_k_out_graph
-from networkx.generators.directed import scale_free_graph
+from networkx.classes import Graph, MultiDiGraph
+from networkx.generators.directed import (
+    gn_graph,
+    gnc_graph,
+    gnr_graph,
+    random_k_out_graph,
+    random_uniform_k_out_graph,
+    scale_free_graph,
+)
 
 
-class TestGeneratorsDirected(object):
+class TestGeneratorsDirected:
     def test_smoke_test_random_graphs(self):
         gn_graph(100)
         gnr_graph(100, 0.5)
@@ -27,14 +28,10 @@ class TestGeneratorsDirected(object):
         scale_free_graph(100, seed=42)
 
     def test_create_using_keyword_arguments(self):
-        pytest.raises(nx.NetworkXError,
-                      gn_graph, 100, create_using=Graph())
-        pytest.raises(nx.NetworkXError,
-                      gnr_graph, 100, 0.5, create_using=Graph())
-        pytest.raises(nx.NetworkXError,
-                      gnc_graph, 100, create_using=Graph())
-        pytest.raises(nx.NetworkXError,
-                      scale_free_graph, 100, create_using=Graph())
+        pytest.raises(nx.NetworkXError, gn_graph, 100, create_using=Graph())
+        pytest.raises(nx.NetworkXError, gnr_graph, 100, 0.5, create_using=Graph())
+        pytest.raises(nx.NetworkXError, gnc_graph, 100, create_using=Graph())
+        pytest.raises(nx.NetworkXError, scale_free_graph, 100, create_using=Graph())
         G = gn_graph(100, seed=1)
         MG = gn_graph(100, create_using=MultiDiGraph(), seed=1)
         assert sorted(G.edges()) == sorted(MG.edges())
@@ -45,16 +42,29 @@ class TestGeneratorsDirected(object):
         MG = gnc_graph(100, create_using=MultiDiGraph(), seed=1)
         assert sorted(G.edges()) == sorted(MG.edges())
 
-        G = scale_free_graph(100, alpha=0.3, beta=0.4, gamma=0.3,
-                             delta_in=0.3, delta_out=0.1,
-                             create_using=MultiDiGraph, seed=1)
+        G = scale_free_graph(
+            100,
+            alpha=0.3,
+            beta=0.4,
+            gamma=0.3,
+            delta_in=0.3,
+            delta_out=0.1,
+            initial_graph=nx.cycle_graph(4, create_using=MultiDiGraph),
+            seed=1,
+        )
         pytest.raises(ValueError, scale_free_graph, 100, 0.5, 0.4, 0.3)
         pytest.raises(ValueError, scale_free_graph, 100, alpha=-0.3)
         pytest.raises(ValueError, scale_free_graph, 100, beta=-0.3)
         pytest.raises(ValueError, scale_free_graph, 100, gamma=-0.3)
 
 
-class TestRandomKOutGraph(object):
+@pytest.mark.parametrize("ig", (nx.Graph(), nx.DiGraph([(0, 1)])))
+def test_scale_free_graph_initial_graph_kwarg(ig):
+    with pytest.raises(nx.NetworkXError):
+        scale_free_graph(100, initial_graph=ig)
+
+
+class TestRandomKOutGraph:
     """Unit tests for the
     :func:`~networkx.generators.directed.random_k_out_graph` function.
 
@@ -79,12 +89,13 @@ class TestRandomKOutGraph(object):
         assert nx.number_of_selfloops(G) == 0
 
 
-class TestUniformRandomKOutGraph(object):
+class TestUniformRandomKOutGraph:
     """Unit tests for the
     :func:`~networkx.generators.directed.random_uniform_k_out_graph`
     function.
 
     """
+
     def test_regularity(self):
         """Tests that the generated graph is `k`-out-regular."""
         n = 10
