@@ -38,9 +38,10 @@ def laplacian_spectrum(G, weight="weight"):
     --------
     laplacian_matrix
     """
-    from scipy.linalg import eigvalsh
+    import scipy as sp
+    import scipy.linalg  # call as sp.linalg
 
-    return eigvalsh(nx.laplacian_matrix(G, weight=weight).todense())
+    return sp.linalg.eigvalsh(nx.laplacian_matrix(G, weight=weight).todense())
 
 
 def normalized_laplacian_spectrum(G, weight="weight"):
@@ -69,9 +70,12 @@ def normalized_laplacian_spectrum(G, weight="weight"):
     --------
     normalized_laplacian_matrix
     """
-    from scipy.linalg import eigvalsh
+    import scipy as sp
+    import scipy.linalg  # call as sp.linalg
 
-    return eigvalsh(nx.normalized_laplacian_matrix(G, weight=weight).todense())
+    return sp.linalg.eigvalsh(
+        nx.normalized_laplacian_matrix(G, weight=weight).todense()
+    )
 
 
 def adjacency_spectrum(G, weight="weight"):
@@ -100,9 +104,10 @@ def adjacency_spectrum(G, weight="weight"):
     --------
     adjacency_matrix
     """
-    from scipy.linalg import eigvals
+    import scipy as sp
+    import scipy.linalg  # call as sp.linalg
 
-    return eigvals(nx.adjacency_matrix(G, weight=weight).todense())
+    return sp.linalg.eigvals(nx.adjacency_matrix(G, weight=weight).todense())
 
 
 def modularity_spectrum(G):
@@ -127,12 +132,13 @@ def modularity_spectrum(G):
     .. [1] M. E. J. Newman, "Modularity and community structure in networks",
        Proc. Natl. Acad. Sci. USA, vol. 103, pp. 8577-8582, 2006.
     """
-    from scipy.linalg import eigvals
+    import scipy as sp
+    import scipy.linalg  # call as sp.linalg
 
     if G.is_directed():
-        return eigvals(nx.directed_modularity_matrix(G))
+        return sp.linalg.eigvals(nx.directed_modularity_matrix(G))
     else:
-        return eigvals(nx.modularity_matrix(G))
+        return sp.linalg.eigvals(nx.modularity_matrix(G))
 
 
 def bethe_hessian_spectrum(G, r=None):
@@ -161,6 +167,7 @@ def bethe_hessian_spectrum(G, r=None):
        "Spectral clustering of graphs with the bethe hessian",
        Advances in Neural Information Processing Systems. 2014.
     """
-    from scipy.linalg import eigvalsh
+    import scipy as sp
+    import scipy.linalg  # call as sp.linalg
 
-    return eigvalsh(nx.bethe_hessian_matrix(G, r).todense())
+    return sp.linalg.eigvalsh(nx.bethe_hessian_matrix(G, r).todense())
