@@ -5,6 +5,7 @@ from collections import deque
 from operator import itemgetter
 
 import networkx as nx
+
 from ..utils import arbitrary_element
 
 __all__ = ["cuthill_mckee_ordering", "reverse_cuthill_mckee_ordering"]
@@ -51,7 +52,7 @@ def cuthill_mckee_ordering(G, heuristic=None):
 
     Notes
     -----
-    The optimal solution the the bandwidth reduction is NP-complete [2]_.
+    The optimal solution the bandwidth reduction is NP-complete [2]_.
 
 
     References
@@ -109,7 +110,7 @@ def reverse_cuthill_mckee_ordering(G, heuristic=None):
 
     Notes
     -----
-    The optimal solution the the bandwidth reduction is NP-complete [2]_.
+    The optimal solution the bandwidth reduction is NP-complete [2]_.
 
     References
     ----------
